import { z } from 'zod';

type EventMap = Record<string, z.ZodTypeAny>;
interface EventEmitterOptions<IncomingEvents extends EventMap = EventMap, OutgoingEvents extends EventMap = EventMap> {
    incomingEvents?: IncomingEvents;
    outgoingEvents?: OutgoingEvents;
}
interface SendActionOptions<IncomingEvents extends EventMap, R extends keyof IncomingEvents> {
    timeoutMs?: number;
    intervalMs?: number;
    condition?: (data: z.infer<IncomingEvents[R]>) => boolean;
}
type SendActionArgs<IncomingEvents extends EventMap, OutgoingEvents extends EventMap, K extends keyof OutgoingEvents, R extends keyof IncomingEvents> = {
    event: K;
    data: z.infer<OutgoingEvents[K]>;
    responseEvent: R;
    options?: SendActionOptions<IncomingEvents, R>;
};
interface OnActionOptions<OutgoingEvents extends EventMap, R extends keyof OutgoingEvents> {
    timeoutMs?: number;
    condition?: (data: z.infer<OutgoingEvents[R]>) => boolean;
}
type OnActionArgs<IncomingEvents extends EventMap, OutgoingEvents extends EventMap, K extends keyof IncomingEvents, R extends keyof OutgoingEvents> = {
    event: K;
    responseEvent: R;
    callback: (data: z.infer<IncomingEvents[K]>) => z.infer<OutgoingEvents[R]>;
    options?: OnActionOptions<OutgoingEvents, R>;
} | {
    event: K;
    options?: OnActionOptions<OutgoingEvents, R>;
};
declare class EventEmitter<IncomingEvents extends EventMap, OutgoingEvents extends EventMap> {
    otherWindow: Window;
    targetOrigin: string | string[];
    incomingEvents: IncomingEvents;
    outgoingEvents: OutgoingEvents;
    private listeners;
    constructor(otherWindow: Window, targetOrigin: string | string[], incomingEvents: IncomingEvents, outgoingEvents: OutgoingEvents);
    send<K extends keyof OutgoingEvents>(event: K, data: z.infer<OutgoingEvents[K]>): void;
    on<K extends keyof IncomingEvents>(event: K, callback: (data: z.infer<IncomingEvents[K]>) => void): string;
    sendAction<K extends keyof OutgoingEvents, R extends keyof IncomingEvents>({ event, data, responseEvent, options, }: SendActionArgs<IncomingEvents, OutgoingEvents, K, R>): Promise<z.infer<IncomingEvents[R]>>;
    onAction<K extends keyof IncomingEvents, R extends keyof OutgoingEvents>({ event, options, ...params }: OnActionArgs<IncomingEvents, OutgoingEvents, K, R>): Promise<z.infer<IncomingEvents[K]>>;
    off(id: string): void;
    protected isTargetOrigin(otherOrigin: string): boolean;
}

export { EventEmitter, type EventEmitterOptions, type EventMap, type OnActionArgs, type OnActionOptions, type SendActionArgs, type SendActionOptions };
