import {
  EventEmitter
} from "./chunk-DZDN7ZJ7.js";
import {
  generateRandomString
} from "./chunk-TX3I4DOZ.js";
import {
  __async,
  __spreadProps,
  __spreadValues,
  __superGet
} from "./chunk-YAEOLK2W.js";

// src/handshake/index.ts
import { z } from "zod";

// src/handshake/Parent.ts
var HandshakeParent = class extends EventEmitter {
  constructor(otherWindow, targetOrigin, options) {
    const mergedIncomingEvents = __spreadValues(__spreadValues({}, options == null ? void 0 : options.incomingEvents), HANDSHAKE_EVENTS.fromChild);
    const mergedOutgoingEvents = __spreadValues(__spreadValues({}, options == null ? void 0 : options.outgoingEvents), HANDSHAKE_EVENTS.fromParent);
    super(otherWindow, targetOrigin, mergedIncomingEvents, mergedOutgoingEvents);
    this.isConnected = false;
    this.handshakeOptions = __spreadValues(__spreadValues({}, DEFAULT_HANDSHAKE_OPTIONS), options == null ? void 0 : options.handshakeOptions);
    this.targetOrigin = targetOrigin;
  }
  handshakeWithChild() {
    return __async(this, null, function* () {
      if (this.isConnected) {
        console.log("[server] Already connected to child");
        return;
      }
      const requestVerificationId = generateRandomString();
      yield this._sendAction({
        event: "handshakeRequest",
        data: { requestVerificationId },
        responseEvent: "handshakeResponse",
        options: {
          timeoutMs: this.handshakeOptions.timeoutMs,
          intervalMs: this.handshakeOptions.intervalMs,
          condition: (data) => data.requestVerificationId === requestVerificationId
        }
      });
      this._send("handshakeComplete", {
        requestVerificationId
      });
      this.isConnected = true;
    });
  }
  // Wrap EventEmitter methods, adding handshake event types
  _send(event, data) {
    return super.send(event, data);
  }
  _sendAction(args) {
    return super.sendAction(__spreadProps(__spreadValues({}, args), {
      options: args.options
      // Fixes weird TS behavior when compiling
    }));
  }
};

// src/handshake/index.ts
var DEFAULT_HANDSHAKE_OPTIONS = {
  timeoutMs: 1e4,
  intervalMs: 100
};
var HANDSHAKE_EVENTS = {
  fromChild: {
    handshakeResponse: z.object({
      requestVerificationId: z.string()
    })
  },
  fromParent: {
    handshakeRequest: z.object({
      requestVerificationId: z.string()
    }),
    handshakeComplete: z.object({
      requestVerificationId: z.string()
    })
  }
};

// src/handshake/Child.ts
var HandshakeChild = class _HandshakeChild extends EventEmitter {
  constructor(otherWindow, targetOrigin, options) {
    const mergedIncomingEvents = __spreadValues(__spreadValues({}, options == null ? void 0 : options.incomingEvents), HANDSHAKE_EVENTS.fromParent);
    const mergedOutgoingEvents = __spreadValues(__spreadValues({}, options == null ? void 0 : options.outgoingEvents), HANDSHAKE_EVENTS.fromChild);
    super(otherWindow, targetOrigin, mergedIncomingEvents, mergedOutgoingEvents);
    this.isConnected = false;
    this.handshakeOptions = __spreadValues(__spreadValues({}, DEFAULT_HANDSHAKE_OPTIONS), options == null ? void 0 : options.handshakeOptions);
  }
  handshakeWithParent() {
    return __async(this, null, function* () {
      if (this.isConnected) {
        console.log("[client] Already connected to parent");
        return;
      }
      const { requestVerificationId } = yield this._onAction({
        event: "handshakeRequest",
        callback: (data) => data,
        responseEvent: "handshakeResponse",
        options: {
          timeoutMs: this.handshakeOptions.timeoutMs
        }
      });
      yield this._onAction({
        event: "handshakeComplete",
        options: {
          timeoutMs: this.handshakeOptions.timeoutMs,
          condition: (data) => data.requestVerificationId === requestVerificationId
        }
      });
      this.isConnected = true;
    });
  }
  // Wrap EventEmitter methods, adding handshake event types
  _onAction(args) {
    return __async(this, null, function* () {
      return __superGet(_HandshakeChild.prototype, this, "onAction").call(this, __spreadProps(__spreadValues({}, args), {
        options: args.options
        // Fixes weird TS behavior when compiling
      }));
    });
  }
};

export {
  HandshakeChild,
  HandshakeParent,
  DEFAULT_HANDSHAKE_OPTIONS,
  HANDSHAKE_EVENTS
};
//# sourceMappingURL=chunk-YFJUB7EJ.js.map